@@ -77,6 +77,9 @@
 //! elements will consume elements from the buffer.
 //!
 //! ```
+//! # #[allow(unused_must_use)]
+//! # #[cfg(feature = "std")]
+//! # {
 //! use circular_buffer::CircularBuffer;
 //! use std::io::Read;
 //! use std::io::Write;
@@ -94,6 +97,7 @@
 //! buf.read_to_string(&mut s).expect("failed to read from buffer");
 //! assert_eq!(s, "round");
 //! assert_eq!(buf, b"");
+//! # }
 //! ```
 //!
 //! # Time complexity
@@ -129,6 +133,8 @@
 //! allocate the buffer on the heap. Use a [`Box`](std::boxed) for that:
 //!
 //! ```
+//! # #[cfg(feature = "std")]
+//! # {
 //! use circular_buffer::CircularBuffer;
 //!
 //! let mut buf = CircularBuffer::<4096, u32>::boxed();
@@ -141,6 +147,7 @@
 //!
 //! buf.truncate_back(128);
 //! assert_eq!(buf.len(), 128);
+//! # }
 //! ```
 //!
 //! # `no_std`
@@ -152,7 +159,7 @@
 //!
 //! ```text
 //! [dependencies]
-//! circular-buffer = { version = "0.1", features = [] }
+//! circular-buffer = { version = "0.1", default-features = false }
 //! ```
 //!
 //! When using `no_std` mode, this crate supports heap-allocation features through the [`alloc`
@@ -160,15 +167,25 @@
 //!
 //! ```text
 //! [dependencies]
-//! circular-buffer = { version = "0.1", features = ["alloc"] }
+//! circular-buffer = { version = "0.1", default-features = false, features = ["alloc"] }
 //! ```
 //!
 //! [`no_std` environment]: https://docs.rust-embedded.org/book/intro/no-std.html
+//!
+//! # Cargo feature flags
+//!
+//! * `std`: enables support for the [`std` library](https://doc.rust-lang.org/std/) (enabled by
+//!   default).
+//! * `alloc`: enables support for the [`alloc` crate](https://doc.rust-lang.org/alloc/) (enabled
+//!   by default).
+//! * `embedded-io`: enables implementation for the
+//!   [`embedded_io`](https://docs.rs/embedded-io/) traits.
+//! * `embedded-io-async`: enables implementation for the
+//!   [`embedded_io_async`](https://docs.rs/embedded-io-async) traits.
 
-#![cfg_attr(not(feature = "use_std"), no_std)]
+#![cfg_attr(not(feature = "std"), no_std)]
 
 #![cfg_attr(feature = "unstable", feature(const_maybe_uninit_uninit_array))]
-#![cfg_attr(feature = "unstable", feature(const_trait_impl))]
 #![cfg_attr(feature = "unstable", feature(maybe_uninit_slice))]
 #![cfg_attr(feature = "unstable", feature(maybe_uninit_uninit_array))]
 #![cfg_attr(feature = "unstable", feature(maybe_uninit_write_slice))]
@@ -179,16 +196,19 @@
 
 #![warn(missing_debug_implementations)]
 #![warn(missing_docs)]
-#![warn(pointer_structural_match)]
 #![warn(unreachable_pub)]
 #![warn(unused_qualifications)]
+#![doc(test(attr(deny(warnings))))]
 
 mod drain;
 mod iter;
 
-#[cfg(feature = "use_std")]
+#[cfg(feature = "std")]
 mod io;
 
+#[cfg(any(feature = "embedded-io", feature = "embedded-io-async"))]
+mod embedded_io;
+
 #[cfg(test)]
 mod tests;
 
@@ -212,33 +232,11 @@ pub use crate::iter::IterMut;
 #[cfg(feature = "alloc")]
 extern crate alloc;
 
-#[cfg(all(not(feature = "use_std"), feature = "alloc"))]
+#[cfg(all(not(feature = "std"), feature = "alloc"))]
 use alloc::boxed::Box;
-#[cfg(all(not(feature = "use_std"), feature = "alloc"))]
+#[cfg(all(not(feature = "std"), feature = "alloc"))]
 use alloc::vec::Vec;
 
-#[cfg(feature = "unstable")]
-macro_rules! unstable_const_impl {
-    (
-        $( #[ $meta:meta ] )*
-        impl $( <{ $( $generics:tt )* }> )? const $trait:ident for $type:ty { $( $tt:tt )* }
-    ) => {
-        $(#[$meta])*
-        impl $(<$($generics)*>)? const $trait for $type { $($tt)* }
-    }
-}
-
-#[cfg(not(feature = "unstable"))]
-macro_rules! unstable_const_impl {
-    (
-        $( #[ $meta:meta ] )*
-        impl $( <{ $( $generics:tt )* }> )? const $trait:ident for $type:ty { $( $tt:tt )* }
-    ) => {
-        $(#[$meta])*
-        impl $(<$($generics)*>)? $trait for $type { $($tt)* }
-    }
-}
-
 /// Returns `(x + y) % m` without risk of overflows if `x + y` cannot fit in `usize`.
 ///
 /// `x` and `y` are expected to be less than, or equal to `m`.
@@ -404,7 +402,7 @@ impl<const N: usize, T> CircularBuffer<N, T> {
     ///
     /// ```
     /// use circular_buffer::CircularBuffer;
-    /// let mut buf = CircularBuffer::<16, u32>::new();
+    /// let buf = CircularBuffer::<16, u32>::new();
     /// assert_eq!(buf.capacity(), 16);
     /// ```
     #[inline]
@@ -1766,7 +1764,7 @@ impl<const N: usize, T> CircularBuffer<N, T>
             // and drops them if the guard is dropped.
             //
             // This implementation was highly inspired by the implementation of
-            // `MaybeUninit::write_slice_cloned`
+            // `MaybeUninit::clone_from_slice`
             struct Guard<'a, T> {
                 dst: &'a mut [MaybeUninit<T>],
                 initialized: usize,
@@ -1814,7 +1812,7 @@ impl<const N: usize, T> CircularBuffer<N, T>
 
             let write_len = core::cmp::min(right.len(), other.len());
             #[cfg(feature = "unstable")]
-            MaybeUninit::write_slice_cloned(&mut right[..write_len], &other[..write_len]);
+            MaybeUninit::clone_from_slice(&mut right[..write_len], &other[..write_len]);
             #[cfg(not(feature = "unstable"))]
             write_uninit_slice_cloned(&mut right[..write_len], &other[..write_len]);
 
@@ -1822,7 +1820,7 @@ impl<const N: usize, T> CircularBuffer<N, T>
             debug_assert!(left.len() >= other.len());
             let write_len = other.len();
             #[cfg(feature = "unstable")]
-            MaybeUninit::write_slice_cloned(&mut left[..write_len], other);
+            MaybeUninit::clone_from_slice(&mut left[..write_len], other);
             #[cfg(not(feature = "unstable"))]
             write_uninit_slice_cloned(&mut left[..write_len], other);
 
@@ -1836,7 +1834,7 @@ impl<const N: usize, T> CircularBuffer<N, T>
             let other = &other[other.len() - N..];
             debug_assert_eq!(self.items.len(), other.len());
             #[cfg(feature = "unstable")]
-            MaybeUninit::write_slice_cloned(&mut self.items, other);
+            MaybeUninit::clone_from_slice(&mut self.items, other);
             #[cfg(not(feature = "unstable"))]
             write_uninit_slice_cloned(&mut self.items, other);
 
@@ -1867,12 +1865,10 @@ impl<const N: usize, T> CircularBuffer<N, T>
     }
 }
 
-unstable_const_impl! {
-    impl<{const N: usize, T}> const Default for CircularBuffer<N, T> {
-        #[inline]
-        fn default() -> Self {
-            Self::new()
-        }
+impl<const N: usize, T> Default for CircularBuffer<N, T> {
+    #[inline]
+    fn default() -> Self {
+        Self::new()
     }
 }
 
@@ -1952,27 +1948,23 @@ impl<const N: usize, T> IndexMut<usize> for CircularBuffer<N, T> {
     }
 }
 
-unstable_const_impl! {
-    impl<{const N: usize, T}> const IntoIterator for CircularBuffer<N, T> {
-        type Item = T;
-        type IntoIter = IntoIter<N, T>;
+impl<const N: usize, T> IntoIterator for CircularBuffer<N, T> {
+    type Item = T;
+    type IntoIter = IntoIter<N, T>;
 
-        #[inline]
-        fn into_iter(self) -> Self::IntoIter {
-            IntoIter::new(self)
-        }
+    #[inline]
+    fn into_iter(self) -> Self::IntoIter {
+        IntoIter::new(self)
     }
 }
 
-unstable_const_impl! {
-    impl<{'a, const N: usize, T}> const IntoIterator for &'a CircularBuffer<N, T> {
-        type Item = &'a T;
-        type IntoIter = Iter<'a, T>;
+impl<'a, const N: usize, T> IntoIterator for &'a CircularBuffer<N, T> {
+    type Item = &'a T;
+    type IntoIter = Iter<'a, T>;
 
-        #[inline]
-        fn into_iter(self) -> Self::IntoIter {
-            Iter::new(self)
-        }
+    #[inline]
+    fn into_iter(self) -> Self::IntoIter {
+        Iter::new(self)
     }
 }
 
