@@ -1,7 +1,9 @@
 // Copyright © 2023, 2024 Andrea Corbellini and contributors
 // SPDX-License-Identifier: BSD-3-Clause
 
-#![cfg(test)]
+#![allow(static_mut_refs)]
+
+#![cfg(feature = "std")]
 
 use crate::CircularBuffer;
 use drop_tracker::DropItem;
